/*
 * © 2017 LogMeIn, Inc. All Rights Reserved.
 * All rights reserved.
 * 
 * This software is distributed under the terms and conditions of the
 * LogMeIn SDK License Agreement. Please see file LICENSE for details.
 * 
 * Auto-generated file.
 */


using System;
using System.Collections.Generic;
using LogMeIn.GoToTraining.Api.Common;
using LogMeIn.GoToTraining.Api.Model;


namespace LogMeIn.GoToTraining.Api
{
    /// <summary>
    ///
    /// </summary>
    public class RegistrationsApi
    {
        private string basePath = "https://api.getgo.com/G2T/rest";
        private readonly ApiInvoker apiInvoker = ApiInvoker.GetInstance();

        /// <summary>
        /// Initializes a new instance of the RegistrationsApi class using the
        /// default endpoint base url for the services being accessed.
        /// </summary>
        public RegistrationsApi()
        {
        }

        /// <summary>
        /// Initializes a new instance of the RegistrationsApi class using an endpoint base
        /// url other than the default. Use this constructor only if you should need
        /// to override the default endpoint base url.
        /// </summary>
        /// <param name="basePath">The endpoint base url for the services being accessed.</param>
        public RegistrationsApi(String basePath)
        {
            this.basePath = basePath;
        }

        /// <summary>
        /// Gets the API invoker
        /// </summary>
        public ApiInvoker Invoker
        {
            get { return apiInvoker; }
        }

        /// <summary>
        /// Gets the endpoint base url for the services being accessed
        /// </summary>
        public String BasePath
        {
            get { return basePath; }
        }

        /// <summary>
        /// Cancel Registration
        /// This call cancels a registration in a scheduled training for a specific registrant. If the registrant has paid for the training, a cancellation cannot be completed with this method; it must be completed on the external admin site. No notification is sent to the registrant or the organizer by default. The registrant can re-register if needed.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <param name="registrantKey">The key of the registrant</param>
        /// <returns>void</returns>
        /// <exception cref="ApiException"></exception>
        public void cancelRegistration(string authorization, long? organizerKey, long? trainingKey, long? registrantKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            if (registrantKey == null)
                throw new ApiException("Required parameter registrantKey is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}/registrants/{registrantKey}"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()))
                .Replace("{" + "registrantKey" + "}", apiInvoker.EscapeString(registrantKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            apiInvoker.InvokeAPI(basePath, path, "DELETE", queryParams, null, headerParams, formParams, typeof(void), contentType);
        }

        /// <summary>
        /// Get Registrant
        /// Retrieves details for specific registrant in a specific training. Registrants can be:<para/>WAITING - registrant registered and is awaiting approval (where organizer has required approval)<para/>APPROVED - registrant registered and is approved<para/>DENIED - registrant registered and was not approved.<para/><para/>IMPORTANT: The registrant data caches are typically updated immediately and the data will be returned in the response. However, the update can take as long as two hours.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <param name="registrantKey">The key of the registrant</param>
        /// <returns>Registrant</returns>
        /// <exception cref="ApiException"></exception>
        public Registrant getRegistrant(string authorization, long? organizerKey, long? trainingKey, long? registrantKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            if (registrantKey == null)
                throw new ApiException("Required parameter registrantKey is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}/registrants/{registrantKey}"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()))
                .Replace("{" + "registrantKey" + "}", apiInvoker.EscapeString(registrantKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(Registrant), contentType);
            if (response != null)
                return (Registrant)response;

            return null;
        }

        /// <summary>
        /// Get Training Registrants
        /// Retrieves details on all registrants for a specific training. Registrants can be:<para/>WAITING - registrant registered and is awaiting approval (where organizer has required approval)<para/>APPROVED - registrant registered and is approved<para/>DENIED - registrant registered and was not approved.<para/><para/>IMPORTANT: The registrant data caches are typically updated immediately and the data will be returned in the response. However, the update can take as long as two hours.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <returns>List&lt;Registrant&gt;</returns>
        /// <exception cref="ApiException"></exception>
        public List<Registrant> getRegistrants(string authorization, long? organizerKey, long? trainingKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}/registrants"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(List<Registrant>), contentType);
            if (response != null)
                return (List<Registrant>)response;

            return null;
        }

        /// <summary>
        /// Register for Training
        /// Registers one person, identified by a unique email address, for a training. Approval is automatic unless payment or approval is required. The response contains the Confirmation page URL and Join URL for the registrant. NOTE: If some registrants do not receive a confirmation email, the emails could be getting blocked by their email server due to spam filtering or a grey-listing setting.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <param name="body">The details of the registrant to create</param>
        /// <returns>RegistrantCreated</returns>
        /// <exception cref="ApiException"></exception>
        public RegistrantCreated registerForTraining(string authorization, long? organizerKey, long? trainingKey, RegistrantReqCreate body)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            if (body == null)
                throw new ApiException("Required parameter body is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}/registrants"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "POST", queryParams, body, headerParams, formParams, typeof(RegistrantCreated), contentType);
            if (response != null)
                return (RegistrantCreated)response;

            return null;
        }


    }
}
