/*
 * © 2017 LogMeIn, Inc. All Rights Reserved.
 * All rights reserved.
 * 
 * This software is distributed under the terms and conditions of the
 * LogMeIn SDK License Agreement. Please see file LICENSE for details.
 * 
 * Auto-generated file.
 */


using System;
using System.Collections.Generic;
using LogMeIn.GoToTraining.Api.Common;
using LogMeIn.GoToTraining.Api.Model;


namespace LogMeIn.GoToTraining.Api
{
    /// <summary>
    ///
    /// </summary>
    public class ReportsApi
    {
        private string basePath = "https://api.getgo.com/G2T/rest";
        private readonly ApiInvoker apiInvoker = ApiInvoker.GetInstance();

        /// <summary>
        /// Initializes a new instance of the ReportsApi class using the
        /// default endpoint base url for the services being accessed.
        /// </summary>
        public ReportsApi()
        {
        }

        /// <summary>
        /// Initializes a new instance of the ReportsApi class using an endpoint base
        /// url other than the default. Use this constructor only if you should need
        /// to override the default endpoint base url.
        /// </summary>
        /// <param name="basePath">The endpoint base url for the services being accessed.</param>
        public ReportsApi(String basePath)
        {
            this.basePath = basePath;
        }

        /// <summary>
        /// Gets the API invoker
        /// </summary>
        public ApiInvoker Invoker
        {
            get { return apiInvoker; }
        }

        /// <summary>
        /// Gets the endpoint base url for the services being accessed
        /// </summary>
        public String BasePath
        {
            get { return basePath; }
        }

        /// <summary>
        /// Get Attendance Details
        /// This call retrieves a list of registrants from a specific completed training session. The response includes the registrants' email addresses, and if they attended, it includes the duration of each period of their attendance in minutes, and the times at which they joined and left. If a registrant does not attend, they appear at the bottom of the listing with timeInSession = 0.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="sessionKey">The key of the training session</param>
        /// <returns>List&lt;Attendee&gt;</returns>
        /// <exception cref="ApiException"></exception>
        public List<Attendee> getAttendanceDetails(string authorization, long? organizerKey, long? sessionKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (sessionKey == null)
                throw new ApiException("Required parameter sessionKey is null.");

            // create path and map variables
            var path = "/reports/organizers/{organizerKey}/sessions/{sessionKey}/attendees"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "sessionKey" + "}", apiInvoker.EscapeString(sessionKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(List<Attendee>), contentType);
            if (response != null)
                return (List<Attendee>)response;

            return null;
        }

        /// <summary>
        /// Get Sessions by Date Range
        /// This call returns all session details over a given date range for a given organizer. A session is a completed training event.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="body">The start and end times for the time range over which to retrieve training sessions</param>
        /// <returns>List&lt;Session&gt;</returns>
        /// <exception cref="ApiException"></exception>
        public List<Session> getSessionDetailsForDateRange(string authorization, long? organizerKey, DateTimeRange body)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (body == null)
                throw new ApiException("Required parameter body is null.");

            // create path and map variables
            var path = "/reports/organizers/{organizerKey}/sessions"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "POST", queryParams, body, headerParams, formParams, typeof(List<Session>), contentType);
            if (response != null)
                return (List<Session>)response;

            return null;
        }

        /// <summary>
        /// Get Sessions By Training
        /// This call returns session details for a given training. A session is a completed training event. Each training may contain one or more sessions.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <returns>List&lt;Session&gt;</returns>
        /// <exception cref="ApiException"></exception>
        public List<Session> getSessionDetailsForTraining(string authorization, long? organizerKey, long? trainingKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            // create path and map variables
            var path = "/reports/organizers/{organizerKey}/trainings/{trainingKey}"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(List<Session>), contentType);
            if (response != null)
                return (List<Session>)response;

            return null;
        }


    }
}
