/*
 * © 2017 LogMeIn, Inc. All Rights Reserved.
 * All rights reserved.
 * 
 * This software is distributed under the terms and conditions of the
 * LogMeIn SDK License Agreement. Please see file LICENSE for details.
 * 
 * Auto-generated file.
 */


using System;
using System.Collections.Generic;
using LogMeIn.GoToTraining.Api.Common;
using LogMeIn.GoToTraining.Api.Model;


namespace LogMeIn.GoToTraining.Api
{
    /// <summary>
    ///
    /// </summary>
    public class TrainingsApi
    {
        private string basePath = "https://api.getgo.com/G2T/rest";
        private readonly ApiInvoker apiInvoker = ApiInvoker.GetInstance();

        /// <summary>
        /// Initializes a new instance of the TrainingsApi class using the
        /// default endpoint base url for the services being accessed.
        /// </summary>
        public TrainingsApi()
        {
        }

        /// <summary>
        /// Initializes a new instance of the TrainingsApi class using an endpoint base
        /// url other than the default. Use this constructor only if you should need
        /// to override the default endpoint base url.
        /// </summary>
        /// <param name="basePath">The endpoint base url for the services being accessed.</param>
        public TrainingsApi(String basePath)
        {
            this.basePath = basePath;
        }

        /// <summary>
        /// Gets the API invoker
        /// </summary>
        public ApiInvoker Invoker
        {
            get { return apiInvoker; }
        }

        /// <summary>
        /// Gets the endpoint base url for the services being accessed
        /// </summary>
        public String BasePath
        {
            get { return basePath; }
        }

        /// <summary>
        /// Delete Training
        /// Deletes a scheduled or completed training. For scheduled trainings, it deletes all scheduled sessions of the training. For completed trainings, the sessions remain in the database. No email is sent to organizers or registrants, but when participants attempt to start or join the training, they are directed to a page that states: Training Not Found: The training you are trying to join is no longer available.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <returns>void</returns>
        /// <exception cref="ApiException"></exception>
        public void cancelTraining(string authorization, long? organizerKey, long? trainingKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            apiInvoker.InvokeAPI(basePath, path, "DELETE", queryParams, null, headerParams, formParams, typeof(void), contentType);
        }

        /// <summary>
        /// Get Trainings
        /// This call retrieves information on all scheduled trainings for a given organizer. The trainings are returned in the order in which they were created. Completed trainings are not included; ongoing trainings with past sessions are included along with the past sessions. If the organizer does not have any scheduled trainings, the response will be empty.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <returns>List&lt;Training&gt;</returns>
        /// <exception cref="ApiException"></exception>
        public List<Training> getAllTrainings(string authorization, long? organizerKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(List<Training>), contentType);
            if (response != null)
                return (List<Training>)response;

            return null;
        }

        /// <summary>
        /// Get Management URL for Training
        /// A request for a direct URL to the admin portal for a specific training. The request identifies the organizer and the training; the response provides a link the organizer can use to manage or launch the training in the admin portal. The training organizer will be required to log in. You can schedule and manage the training (e.g., add tests, polls and training materials) from the URL provided in the response.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <returns>string</returns>
        /// <exception cref="ApiException"></exception>
        public string getManageTrainingURL(string authorization, long? organizerKey, long? trainingKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}/manageUrl"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(string), contentType);
            if (response != null)
                return (string)response;

            return null;
        }

        /// <summary>
        /// Get Training Organizers
        /// Retrieves organizer details for a specific training. This is only applicable to multi-user accounts with sharing enabled (co-organizers).
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <returns>List&lt;Organizer&gt;</returns>
        /// <exception cref="ApiException"></exception>
        public List<Organizer> getOrganisersForTraining(string authorization, long? organizerKey, long? trainingKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}/organizers"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(List<Organizer>), contentType);
            if (response != null)
                return (List<Organizer>)response;

            return null;
        }

        /// <summary>
        /// Get Start Url
        /// Returns a URL that can be used to start a training. When this URL is opened in a web browser, the GoToTraining client will be downloaded and launched and the training will start after the organizer logs in with its credentials.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <returns>string</returns>
        /// <exception cref="ApiException"></exception>
        public string getStartUrl(string authorization, long? organizerKey, long? trainingKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}/startUrl"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(string), contentType);
            if (response != null)
                return (string)response;

            return null;
        }

        /// <summary>
        /// Get Training
        /// Uses the organizer key and training key to retrieve information on a scheduled training.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <returns>Training</returns>
        /// <exception cref="ApiException"></exception>
        public Training getTraining(string authorization, long? organizerKey, long? trainingKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(Training), contentType);
            if (response != null)
                return (Training)response;

            return null;
        }

        /// <summary>
        /// Create Training
        /// Schedules a training of one or more sessions. The call requires a training's name, at least one start and end time, and optionally may include additional sessions, a description, additional organizers (presenters), and registration settings. You can only add organizers to a training if you have a multi-user account. Once a training has been created with this method, you can accept registrations to the training. Registration is for the entire training - all sessions. (The GoToTraining admin site enables you to create trainings that allow participants to register for individual sessions as well as automatically create weekly or monthly events.) Registration settings controls whether you allow web registration for this training, and whether a confirmation email is sent to the registrant following registration. Disabling the confirmation email is an API-only setting. If the user registers through the GoToTraining website, a confirmation email is sent. If the user is manually approved by the training administrator through the GoToTraining web site, the confirmation email is sent. It is recommended that you disable web registration if you disable confirmation emails. The response contains a trainingKey for the scheduled training.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="body">The details of the training to create</param>
        /// <returns>string</returns>
        /// <exception cref="ApiException"></exception>
        public string scheduleTraining(string authorization, long? organizerKey, TrainingReqCreate body)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (body == null)
                throw new ApiException("Required parameter body is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "POST", queryParams, body, headerParams, formParams, typeof(string), contentType);
            if (response != null)
                return (string)response;

            return null;
        }

        /// <summary>
        /// Start Training
        /// Returns a URL that can be used to start a training. When this URL is opened in a web browser, the GoToTraining client will be downloaded and launched and the training will start. A login of the organizer is not required.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <returns>HostUrl</returns>
        /// <exception cref="ApiException"></exception>
        public HostUrl startTraining(string authorization, long? trainingKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            // create path and map variables
            var path = "/trainings/{trainingKey}/start"
                .Replace("{format}", "json")
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(HostUrl), contentType);
            if (response != null)
                return (HostUrl)response;

            return null;
        }

        /// <summary>
        /// Update Training Organizers
        /// Replaces the co-organizers for a specific training. The scheduling organizer cannot be unassigned. Organizers will be notified via email if the notifyOrganizers parameter is set to true. Replaced organizers are not notified. This method is only applicable to multi-user accounts with sharing enabled (co-organizers).
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <param name="body">The details of the training to create</param>
        /// <returns>void</returns>
        /// <exception cref="ApiException"></exception>
        public void updateOrganisersForTraining(string authorization, long? organizerKey, long? trainingKey, TrainingOrganizers body)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            if (body == null)
                throw new ApiException("Required parameter body is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}/organizers"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            apiInvoker.InvokeAPI(basePath, path, "PUT", queryParams, body, headerParams, formParams, typeof(void), contentType);
        }

        /// <summary>
        /// Update Training Registration Settings
        /// An API request to automatically enable or disable web registrations and confirmation emails to registrants.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <param name="body">The new registration settings for the training</param>
        /// <returns>void</returns>
        /// <exception cref="ApiException"></exception>
        public void updateRegistrationSettingsForTraining(string authorization, long? organizerKey, long? trainingKey, RegistrationSettings body)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            if (body == null)
                throw new ApiException("Required parameter body is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}/registrationSettings"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            apiInvoker.InvokeAPI(basePath, path, "PUT", queryParams, body, headerParams, formParams, typeof(void), contentType);
        }

        /// <summary>
        /// Update Training Name and Description
        /// Request to update a scheduled training name and description.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <param name="body">The new name and description for the training</param>
        /// <returns>void</returns>
        /// <exception cref="ApiException"></exception>
        public void updateTrainingNameDescription(string authorization, long? organizerKey, long? trainingKey, TrainingNameDescription body)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            if (body == null)
                throw new ApiException("Required parameter body is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}/nameDescription"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            apiInvoker.InvokeAPI(basePath, path, "PUT", queryParams, body, headerParams, formParams, typeof(void), contentType);
        }

        /// <summary>
        /// Update Training Times
        ///  A request to update a scheduled training's start and end times. If the request contains 'notifyTrainers = true' and 'notifyRegistrants = true', both organizers and registrants are notified. The response provides the number of notified trainers and registrants.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="organizerKey">The key of the training organizer</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <param name="body">The new start and end times for the scheduled training</param>
        /// <returns>NotifiedParties</returns>
        /// <exception cref="ApiException"></exception>
        public NotifiedParties updateTrainingTimes(string authorization, long? organizerKey, long? trainingKey, TrainingTimes body)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (organizerKey == null)
                throw new ApiException("Required parameter organizerKey is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            if (body == null)
                throw new ApiException("Required parameter body is null.");

            // create path and map variables
            var path = "/organizers/{organizerKey}/trainings/{trainingKey}/times"
                .Replace("{format}", "json")
                .Replace("{" + "organizerKey" + "}", apiInvoker.EscapeString(organizerKey.ToString()))
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "PUT", queryParams, body, headerParams, formParams, typeof(NotifiedParties), contentType);
            if (response != null)
                return (NotifiedParties)response;

            return null;
        }


    }
}
